@@ -67,20 +67,38 @@ namespace ojph {
       allocator->pre_alloc_obj<ui32>(num_comps); //for line_offsets
       allocator->pre_alloc_obj<ui32>(num_comps); //for num_bits
       allocator->pre_alloc_obj<bool>(num_comps); //for is_signed
-      allocator->pre_alloc_obj<bool>(num_comps); //for nlt_type3
+      allocator->pre_alloc_obj<bool>(num_comps); //for reversible
+      allocator->pre_alloc_obj<ui8>(num_comps);  //for nlt_type3
       allocator->pre_alloc_obj<ui32>(num_comps); //for cur_line
 
-      ui32 tilepart_div = codestream->get_tilepart_div();
-      num_tileparts = 1; //for num_rc_bytes
-      // this code is not ideal, since the number of decompositions can be 
-      // different for different components
-      if (tilepart_div & OJPH_TILEPART_COMPONENTS)
-        num_tileparts *= num_comps;
-      if (tilepart_div & OJPH_TILEPART_RESOLUTIONS)
-        num_tileparts *= codestream->get_cod()->get_num_decompositions() + 1;
-      if (num_tileparts > 255)
-        OJPH_ERROR(0x000300D1, "Trying to create %d tileparts; a tile cannot "
-          "have more than 255 tile parts.", num_tileparts);
+      {
+        ui32 tilepart_div = codestream->get_tilepart_div();
+        ui32 t = tilepart_div & OJPH_TILEPART_MASK;
+        if (t == OJPH_TILEPART_NO_DIVISIONS)
+          num_tileparts = 1; //for num_rc_bytes
+        else if (t == OJPH_TILEPART_COMPONENTS)
+          num_tileparts = num_comps;
+        else if (t == OJPH_TILEPART_RESOLUTIONS)
+        {
+          ui32 max_decs = 0;
+          for (ui32 c = 0; c < num_comps; ++c) {
+            ui32 s = codestream->get_coc(c)->get_num_decompositions();
+            max_decs = ojph_max(max_decs, s);
+          }
+          num_tileparts = 1 + max_decs;
+        }
+        else if (t == (OJPH_TILEPART_COMPONENTS | OJPH_TILEPART_RESOLUTIONS))
+        {
+          num_tileparts = 0;
+          for (ui32 c = 0; c < num_comps; ++c) {
+            ui32 s = codestream->get_coc(c)->get_num_decompositions();
+            num_tileparts += s + 1;
+          }
+        }
+        if (num_tileparts > 255)
+          OJPH_ERROR(0x000300D1, "Trying to create %d tileparts; a tile "
+            "cannot have more than 255 tile parts.", num_tileparts);
+      }
 
       ui32 tx0 = tile_rect.org.x;
       ui32 ty0 = tile_rect.org.y;
@@ -122,11 +140,30 @@ namespace ojph {
       }
 
       //allocate lines
-      if (codestream->get_cod()->is_employing_color_transform())
+      const param_cod* cdp = codestream->get_cod();
+      if (cdp->is_employing_color_transform())
       {
+        bool reversible[3];
+        for (ui32 i = 0; i < 3; ++i)
+          reversible[i] = codestream->get_coc(i)->is_reversible();
+        if (reversible[0] != reversible[1] || reversible[1] != reversible[2])
+          OJPH_ERROR(0x000300A2, "When the colour transform is employed. "
+            "all colour components must undergo either reversible or "
+            "irreversible wavelet transform; if not, then it is not clear "
+            "what colour transform should be used (reversible or "
+            "irreversible).  Here we found that the first three colour "
+            "components uses %s, %s, and %s transforms, respectively.",
+            reversible[0] ? "reversible" : "irreversible",
+            reversible[1] ? "reversible" : "irreversible",
+            reversible[2] ? "reversible" : "irreversible");
+
         allocator->pre_alloc_obj<line_buf>(3);
-        for (int i = 0; i < 3; ++i)
-          allocator->pre_alloc_data<si32>(width, 0);
+        if (reversible[0])
+          for (int i = 0; i < 3; ++i)
+            allocator->pre_alloc_data<si32>(width, 0);
+        else
+          for (int i = 0; i < 3; ++i)
+            allocator->pre_alloc_data<float>(width, 0);
       }
     }
 
@@ -154,19 +191,41 @@ namespace ojph {
       line_offsets = allocator->post_alloc_obj<ui32>(num_comps);
       num_bits = allocator->post_alloc_obj<ui32>(num_comps);
       is_signed = allocator->post_alloc_obj<bool>(num_comps);
-      nlt_type3 = allocator->post_alloc_obj<bool>(num_comps);
+      reversible = allocator->post_alloc_obj<bool>(num_comps);
+      nlt_type3 = allocator->post_alloc_obj<ui8>(num_comps);
       cur_line = allocator->post_alloc_obj<ui32>(num_comps);
 
       profile = codestream->get_profile();
       tilepart_div = codestream->get_tilepart_div();
       need_tlm = codestream->is_tlm_needed();
-      num_tileparts = 1;
-      // this code is not ideal, since the number of decompositions can be 
-      // different for different components
-      if (tilepart_div & OJPH_TILEPART_COMPONENTS)
-        num_tileparts *= num_comps;
-      if (tilepart_div & OJPH_TILEPART_RESOLUTIONS)
-        num_tileparts *= codestream->get_cod()->get_num_decompositions() + 1;
+      {
+        ui32 tilepart_div = codestream->get_tilepart_div();
+        ui32 t = tilepart_div & OJPH_TILEPART_MASK;
+        if (t == OJPH_TILEPART_NO_DIVISIONS)
+          num_tileparts = 1; //for num_rc_bytes
+        else if (t == OJPH_TILEPART_COMPONENTS)
+          num_tileparts = num_comps;
+        else if (t == OJPH_TILEPART_RESOLUTIONS)
+        {
+          ui32 max_decs = 0;
+          for (ui32 c = 0; c < num_comps; ++c) {
+            ui32 s = codestream->get_coc(c)->get_num_decompositions();
+            max_decs = ojph_max(max_decs, s);
+          }
+          num_tileparts = 1 + max_decs;
+        }
+        else if (t == (OJPH_TILEPART_COMPONENTS | OJPH_TILEPART_RESOLUTIONS))
+        {
+          num_tileparts = 0;
+          for (ui32 c = 0; c < num_comps; ++c) {
+            ui32 s = codestream->get_coc(c)->get_num_decompositions();
+            num_tileparts += s + 1;
+          }
+        }
+        if (num_tileparts > 255)
+          OJPH_ERROR(0x000300D1, "Trying to create %d tileparts; a tile "
+          "cannot have more than 255 tile parts.", num_tileparts);
+      }
 
       this->resilient = codestream->is_resilient();
       this->tile_rect = tile_rect;
@@ -210,28 +269,34 @@ namespace ojph {
 
         num_bits[i] = szp->get_bit_depth(i);
         is_signed[i] = szp->is_signed(i);
-        nlt_type3[i] = nlp->get_type3_transformation(i, bd, is);
-        if (nlt_type3[i] == true && (bd != num_bits[i] || is != is_signed[i]))
+        bool result = nlp->get_nonlinear_transform(i, bd, is, nlt_type3[i]);
+        if (result == true && (bd != num_bits[i] || is != is_signed[i]))
           OJPH_ERROR(0x000300A1, "Mismatch between Ssiz (bit_depth = %d, "
             "is_signed = %s) from SIZ marker segment, and BDnlt "
             "(bit_depth = %d, is_signed = %s) from NLT marker segment, "
-            "for component %d",i, num_bits[i], 
+            "for component %d", i, num_bits[i], 
             is_signed[i] ? "True" : "False", bd, is ? "True" : "False");
         cur_line[i] = 0;
+        reversible[i] = codestream->get_coc(i)->is_reversible();
       }
 
       offset += tile_rect.siz.w;
 
       //allocate lines
       const param_cod* cdp = codestream->get_cod();
-      this->reversible = cdp->access_atk()->is_reversible();
       this->employ_color_transform = cdp->is_employing_color_transform();
       if (this->employ_color_transform)
       {
         num_lines = 3;
         lines = allocator->post_alloc_obj<line_buf>(num_lines);
-        for (int i = 0; i < 3; ++i)
-          lines[i].wrap(allocator->post_alloc_data<si32>(width, 0), width, 0);
+        if (reversible[0])
+          for (int i = 0; i < 3; ++i)
+            lines[i].wrap(
+              allocator->post_alloc_data<si32>(width, 0), width, 0);
+        else
+          for (int i = 0; i < 3; ++i)
+            lines[i].wrap(
+              allocator->post_alloc_data<float>(width, 0), width, 0);
       }
       else
       {
@@ -244,6 +309,9 @@ namespace ojph {
     //////////////////////////////////////////////////////////////////////////
     bool tile::push(line_buf *line, ui32 comp_num)
     {
+      constexpr ui8 type3 = 
+        param_nlt::nonlinearity::OJPH_NLT_BINARY_COMPLEMENT_NLT;
+
       assert(comp_num < num_comps);
       if (cur_line[comp_num] >= comp_rects[comp_num].siz.h)
         return false;
@@ -256,10 +324,10 @@ namespace ojph {
         assert(comp_num < num_comps);
         ui32 comp_width = comp_rects[comp_num].siz.w;
         line_buf *tc = comps[comp_num].get_line();
-        if (reversible)
+        if (reversible[comp_num])
         {
           si64 shift = (si64)1 << (num_bits[comp_num] - 1);
-          if (is_signed[comp_num] && nlt_type3[comp_num])
+          if (is_signed[comp_num] && nlt_type3[comp_num] == type3)
             rev_convert_nlt_type3(line, line_offsets[comp_num],
               tc, 0, shift + 1, comp_width);
           else {
@@ -270,13 +338,12 @@ namespace ojph {
         }
         else
         {
-          float mul = 1.0f / (float)(1<<num_bits[comp_num]);
-          const si32 *sp = line->i32 + line_offsets[comp_num];
-          float *dp = tc->f32;
-          if (is_signed[comp_num])
-            cnvrt_si32_to_float(sp, dp, mul, comp_width);
+          if (nlt_type3[comp_num] == type3)
+            irv_convert_to_float_nlt_type3(line, line_offsets[comp_num],
+              tc, num_bits[comp_num], is_signed[comp_num], comp_width);
           else
-            cnvrt_si32_to_float_shftd(sp, dp, mul, comp_width);
+            irv_convert_to_float(line, line_offsets[comp_num],
+              tc, num_bits[comp_num], is_signed[comp_num], comp_width);
         }
         comps[comp_num].push_line();
       }
@@ -284,9 +351,9 @@ namespace ojph {
       {
         si64 shift = (si64)1 << (num_bits[comp_num] - 1);
         ui32 comp_width = comp_rects[comp_num].siz.w;
-        if (reversible)
+        if (reversible[comp_num])
         {
-          if (is_signed[comp_num] && nlt_type3[comp_num])
+          if (is_signed[comp_num] && nlt_type3[comp_num] == type3)
             rev_convert_nlt_type3(line, line_offsets[comp_num], 
               lines + comp_num, 0, shift + 1, comp_width);            
           else {
@@ -308,13 +375,14 @@ namespace ojph {
         }
         else
         {
-          float mul = 1.0f / (float)(1<<num_bits[comp_num]);
-          const si32 *sp = line->i32 + line_offsets[comp_num];
-          float *dp = lines[comp_num].f32;
-          if (is_signed[comp_num])
-            cnvrt_si32_to_float(sp, dp, mul, comp_width);
+          if (nlt_type3[comp_num] == type3)
+            irv_convert_to_float_nlt_type3(line, line_offsets[comp_num],
+              lines + comp_num, num_bits[comp_num], is_signed[comp_num], 
+              comp_width);
           else
-            cnvrt_si32_to_float_shftd(sp, dp, mul, comp_width);
+            irv_convert_to_float(line, line_offsets[comp_num],
+              lines + comp_num, num_bits[comp_num], is_signed[comp_num], 
+              comp_width);
           if (comp_num == 2)
           { // irreversible color transform
             ict_forward(lines[0].f32, lines[1].f32, lines[2].f32,
@@ -334,6 +402,9 @@ namespace ojph {
     //////////////////////////////////////////////////////////////////////////
     bool tile::pull(line_buf* tgt_line, ui32 comp_num)
     {
+      constexpr ui8 type3 = 
+        param_nlt::nonlinearity::OJPH_NLT_BINARY_COMPLEMENT_NLT;
+
       assert(comp_num < num_comps);
       if (cur_line[comp_num] >= recon_comp_rects[comp_num].siz.h)
         return false;
@@ -344,10 +415,10 @@ namespace ojph {
       {
         line_buf *src_line = comps[comp_num].pull_line();
         ui32 comp_width = recon_comp_rects[comp_num].siz.w;
-        if (reversible)
+        if (reversible[comp_num])
         {
           si64 shift = (si64)1 << (num_bits[comp_num] - 1);
-          if (is_signed[comp_num] && nlt_type3[comp_num])
+          if (is_signed[comp_num] && nlt_type3[comp_num] == type3)
             rev_convert_nlt_type3(src_line, 0, tgt_line, 
               line_offsets[comp_num], shift + 1, comp_width);
           else {
@@ -358,13 +429,14 @@ namespace ojph {
         }
         else
         {
-          float mul = (float)(1 << num_bits[comp_num]);
-          const float *sp = src_line->f32;
-          si32 *dp = tgt_line->i32 + line_offsets[comp_num];
-          if (is_signed[comp_num])
-            cnvrt_float_to_si32(sp, dp, mul, comp_width);
+          if (nlt_type3[comp_num] == type3)
+            irv_convert_to_integer_nlt_type3(src_line, tgt_line, 
+              line_offsets[comp_num], num_bits[comp_num], 
+              is_signed[comp_num], comp_width);
           else
-            cnvrt_float_to_si32_shftd(sp, dp, mul, comp_width);
+            irv_convert_to_integer(src_line, tgt_line, 
+              line_offsets[comp_num], num_bits[comp_num], 
+              is_signed[comp_num], comp_width);
         }
       }
       else
@@ -373,7 +445,7 @@ namespace ojph {
         ui32 comp_width = recon_comp_rects[comp_num].siz.w;
         if (comp_num == 0)
         {
-          if (reversible)
+          if (reversible[comp_num])
             rct_backward(comps[0].pull_line(), comps[1].pull_line(),
               comps[2].pull_line(), lines + 0, lines + 1,
               lines + 2, comp_width);
@@ -382,7 +454,7 @@ namespace ojph {
               comps[2].pull_line()->f32, lines[0].f32, lines[1].f32,
               lines[2].f32, comp_width);
         }
-        if (reversible)
+        if (reversible[comp_num])
         {
           si64 shift = (si64)1 << (num_bits[comp_num] - 1);
           line_buf* src_line;
@@ -390,7 +462,7 @@ namespace ojph {
             src_line = lines + comp_num;
           else
             src_line = comps[comp_num].pull_line();
-          if (is_signed[comp_num] && nlt_type3[comp_num])
+          if (is_signed[comp_num] && nlt_type3[comp_num] == type3)
             rev_convert_nlt_type3(src_line, 0, tgt_line, 
               line_offsets[comp_num], shift + 1, comp_width);
           else {
@@ -401,17 +473,19 @@ namespace ojph {
         }
         else
         {
-          float mul = (float)(1 << num_bits[comp_num]);
-          const float *sp;
+          line_buf* lbp;
           if (comp_num < 3)
-            sp = lines[comp_num].f32;
+            lbp = lines + comp_num;
           else
-            sp = comps[comp_num].pull_line()->f32;
-          si32 *dp = tgt_line->i32 + line_offsets[comp_num];
-          if (is_signed[comp_num])
-            cnvrt_float_to_si32(sp, dp, mul, comp_width);
+            lbp = comps[comp_num].pull_line();            
+          if (nlt_type3[comp_num] == type3)
+            irv_convert_to_integer_nlt_type3(lbp, tgt_line, 
+              line_offsets[comp_num], num_bits[comp_num], 
+              is_signed[comp_num], comp_width);
           else
-            cnvrt_float_to_si32_shftd(sp, dp, mul, comp_width);
+            irv_convert_to_integer(lbp, tgt_line, 
+              line_offsets[comp_num], num_bits[comp_num], 
+              is_signed[comp_num], comp_width);
         }
       }
 
